@@ -1,7 +1,7 @@
 /*
     net_socket.c -- Handle various kinds of sockets.
     Copyright (C) 1998-2005 Ivo Timmermans,
-                  2000-2010 Guus Sliepen <guus@tinc-vpn.org>
+                  2000-2012 Guus Sliepen <guus@tinc-vpn.org>
                   2006      Scott Lamb <slamb@slamb.org>
                   2009      Florian Forster <octo@verplant.org>
 
@@ -34,8 +34,6 @@
 #include "utils.h"
 #include "xalloc.h"
 
-#include <assert.h>
-
 /* Needed on Mac OS/X */
 #ifndef SOL_TCP
 #define SOL_TCP IPPROTO_TCP
@@ -110,63 +108,6 @@ static bool bind_to_interface(int sd) {
 	return true;
 }
 
-static bool bind_to_address(connection_t *c) {
-	char *node;
-	struct addrinfo *ai_list;
-	struct addrinfo *ai_ptr;
-	struct addrinfo ai_hints;
-	int status;
-
-	assert(c != NULL);
-	assert(c->socket >= 0);
-
-	node = NULL;
-	if(!get_config_string(lookup_config(config_tree, "BindToAddress"),
-				&node))
-		return true;
-
-	assert(node != NULL);
-
-	memset(&ai_hints, 0, sizeof(ai_hints));
-	ai_hints.ai_family = c->address.sa.sa_family;
-	/* We're called from `do_outgoing_connection' only. */
-	ai_hints.ai_socktype = SOCK_STREAM;
-	ai_hints.ai_protocol = IPPROTO_TCP;
-
-	ai_list = NULL;
-
-	status = getaddrinfo(node, /* service = */ NULL,
-			&ai_hints, &ai_list);
-	if(status) {
-		free(node);
-		logger(LOG_WARNING, "Error looking up %s port %s: %s",
-				node, "any", gai_strerror(status));
-		return false;
-	}
-	assert(ai_list != NULL);
-
-	status = -1;
-	for(ai_ptr = ai_list; ai_ptr != NULL; ai_ptr = ai_ptr->ai_next) {
-		status = bind(c->socket,
-				ai_list->ai_addr, ai_list->ai_addrlen);
-		if(!status)
-			break;
-	}
-
-
-	if(status) {
-		logger(LOG_ERR, "Can't bind to %s/tcp: %s", node, sockstrerror(sockerrno));
-	} else ifdebug(CONNECTIONS) {
-		logger(LOG_DEBUG, "Successfully bound outgoing "
-				"TCP socket to %s", node);
-	}
-
-	free(node);
-	freeaddrinfo(ai_list);
-
-	return status ? false : true;
-}
-
 int setup_listen_socket(const sockaddr_t *sa) {
 	int nfd;
 	char *addrstr;
@@ -180,6 +121,10 @@ int setup_listen_socket(const sockaddr_t *sa) {
 		return -1;
 	}
 
+#ifdef FD_CLOEXEC
+	fcntl(nfd, F_SETFD, FD_CLOEXEC);
+#endif
+
 	/* Optimize TCP settings */
 
 	option = 1;
@@ -238,6 +183,10 @@ int setup_vpn_in_socket(const sockaddr_t *sa) {
 		return -1;
 	}
 
+#ifdef FD_CLOEXEC
+	fcntl(nfd, F_SETFD, FD_CLOEXEC);
+#endif
+
 #ifdef O_NONBLOCK
 	{
 		int flags = fcntl(nfd, F_GETFL);
@@ -262,6 +211,7 @@ int setup_vpn_in_socket(const sockaddr_t *sa) {
 
 	option = 1;
 	setsockopt(nfd, SOL_SOCKET, SO_REUSEADDR, (void *)&option, sizeof(option));
+	setsockopt(nfd, SOL_SOCKET, SO_BROADCAST, (void *)&option, sizeof(option));
 
 	if(udp_rcvbuf && setsockopt(nfd, SOL_SOCKET, SO_RCVBUF, (void *)&udp_rcvbuf, sizeof(udp_rcvbuf)))
 		logger(LOG_WARNING, "Can't set UDP SO_RCVBUF to %i: %s", udp_rcvbuf, strerror(errno));
@@ -410,6 +360,10 @@ begin:
 
 	c->socket = socket(c->address.sa.sa_family, SOCK_STREAM, IPPROTO_TCP);
 
+#ifdef FD_CLOEXEC
+	fcntl(c->socket, F_SETFD, FD_CLOEXEC);
+#endif
+
 	if(c->socket == -1) {
 		ifdebug(CONNECTIONS) logger(LOG_ERR, "Creating socket for %s failed: %s", c->hostname, sockstrerror(sockerrno));
 		goto begin;
@@ -422,7 +376,6 @@ begin:
 #endif
 
 	bind_to_interface(c->socket);
-	bind_to_address(c);
 
 	/* Optimize TCP settings */
 
