@@ -1,7 +1,7 @@
 /*
     protocol.c -- handle the meta-protocol, basic functions
     Copyright (C) 1999-2005 Ivo Timmermans,
-                  2000-2009 Guus Sliepen <guus@tinc-vpn.org>
+                  2000-2012 Guus Sliepen <guus@tinc-vpn.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -96,7 +96,7 @@ bool send_request(connection_t *c, const char *format, ...) {
 
 	buffer[len++] = '\n';
 
-	if(c == broadcast) {
+	if(c == everyone) {
 		broadcast_meta(NULL, buffer, len);
 		return true;
 	} else
